<?php

namespace Pqc\Coin\Providers;

use Illuminate\Support\ServiceProvider;
use Pqc\Coin\Coins;

class CoinServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = true;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        // Load view
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'coin');

        // Load translation
        $this->loadTranslationsFrom(__DIR__ . '/../../resources/lang', 'coin');

        // Load migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');

        // Call pblish redources function
        $this->publishResources();

    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfig();
        $this->registerFacade();
        // $this->registerCommands();

        $this->app->register(\Pqc\Coin\Providers\AuthServiceProvider::class);
        $this->app->register(\Pqc\Coin\Providers\RouteServiceProvider::class);
        // $this->app->register(\Pqc\Coin\Providers\EventServiceProvider::class);
        // $this->app->register(\Pqc\Coin\Providers\WorkflowServiceProvider::class);
    }

    /**
     * Register the vault facade without the user having to add it to the app.php file.
     *
     * @return void
     */
    public function registerFacade() {
        $this->app->bind('pqc.coin', function($app)
        {
            return $this->app->make(Coins::class);
        });
    }



    /**
     * Merges user's and coin's configs.
     *
     * @return void
     */
    protected function mergeConfig()
    {
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/config.php', 'pqc.coin'
        );
        
        
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/master.php', 'pqc.coin.master'
        );
    }

    /**
     * Register scaffolding command
     */
    protected function registerCommands()
    {
        if ($this->app->runningInConsole()) {
            $this->commands([
                \Pqc\Coin\Commands\Coin::class,
            ]);
        }
    }
    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return ['pqc.coin'];
    }

    /**
     * Publish resources.
     *
     * @return void
     */
    private function publishResources()
    {
        // Publish configuration file
        $this->publishes([__DIR__ . '/../../config/' => config_path('pqc/coin')], 'config');

        // Publish admin view
        $this->publishes([__DIR__ . '/../../resources/views' => base_path('resources/views/vendor/coin')], 'view');

        // Publish language files
        $this->publishes([__DIR__ . '/../../resources/lang' => base_path('resources/lang/vendor/coin')], 'lang');

        // Publish public files and assets.
        $this->publishes([__DIR__ . '/public/' => public_path('/')], 'public');
    }
}
